#include "Debug.h"
#include "led.h"

BYTE mon_rx_buff[MON_BUFF_SIZE];
WORD mon_rx_len = 0;

BYTE mon_tx_buff[MON_BUFF_SIZE];
WORD mon_tx_pos = 0;
WORD mon_tx_end = 0;

BYTE mon_hex_buff[MON_BUFF_SIZE];
WORD mon_hex_len = 0;

BOOL ReceiveStart = FALSE;
BOOL ReceiveOK = FALSE;
BOOL CommandOK = FALSE;
WORD cksum = 0;

BYTE asc2hex(BYTE asc){
	if((asc >= '0') && (asc <= '9')) return (BYTE)(asc - '0');
    else if((asc >= 'A') && (asc <= 'F')) return (BYTE)(asc - 'A' + 0x0A);
    else if((asc >= 'a') && (asc <= 'f')) return (BYTE)(asc - 'a' + 0x0A);
    else return ASC_HEX_ERROR;
}

BYTE hex2asc(BYTE hex){
BYTE da;
	da = (BYTE)(hex & 0x0F);
    if((da >= 0) && (da <= 9)) return (BYTE)('0' + da);
    else return (BYTE)('A' + da - 0x0A);
}

void InitDebug()
{
	// IPR8
	INTC_IPR8 = 0x4040;
	INTC_IPR8 = 0x8080;
	
	// UART 0
	SCI0_SR;
	GPIO_E_PER |= 0x0003;
	SCI0_BR = 98;			// BPS 38400
	SCI0_CR = 0x00AC;
}	

void push_tx_buff(char c)
{
	mon_tx_buff[mon_tx_end++] = c;
}

void push_tx_buff_hex(BYTE h)
{
	mon_tx_buff[mon_tx_end++] = hex2asc(h >> 4);
	mon_tx_buff[mon_tx_end++] = hex2asc(h);
}

#pragma interrupt alignsp
void SCI0_RX_FULL(void)
{
BYTE i, RX, bData;
WORD wData;
DWORD nAddr, dwData;

	RX = (BYTE)SCI0_SR;
	RX = (BYTE)SCI0_DR;
	if(RX == START_CHAR)
	{	
		mon_rx_len = 0;
		mon_hex_len = 0;
		cksum = 0;
		ReceiveStart = TRUE;
	}
	else if(ReceiveStart)
	{
		if(RX != END_CHAR)
		{
			mon_rx_buff[mon_rx_len] = RX;
			if(mon_rx_len >= MON_BUFF_SIZE) ReceiveStart = FALSE;
			
			cksum += (WORD)RX;
			
			if((mon_rx_len % 2) == 0) mon_hex_buff[mon_hex_len] = (WORD)(asc2hex(RX) << 4);
			else
			{
				mon_hex_buff[mon_hex_len] |= asc2hex(RX);
				mon_hex_len++;
			}
			
			mon_rx_len++;
		}
		else
		{
			ReceiveStart = FALSE;			
			cksum -= mon_rx_buff[mon_rx_len - 1];
			cksum -= mon_rx_buff[mon_rx_len - 2];
			if((cksum & 0xFF) == mon_hex_buff[mon_hex_len - 1])
			{				
				nAddr  = (DWORD)(mon_hex_buff[2] << 16);
				nAddr |= (DWORD)(mon_hex_buff[3] << 8);
				nAddr |= (DWORD)(mon_hex_buff[4]);
				
				mon_tx_pos = 0;
				mon_tx_end = 0;
				if(mon_hex_buff[0] == 0x51)		// Read Command
				{
					push_tx_buff(START_CHAR);
					push_tx_buff_hex(0x51);
					
					if(mon_hex_buff[1] == 0)			// 8Bit Low Data Read
					{
						wData = PWORD(nAddr);
						bData = (BYTE)(wData & 0xFF);
						push_tx_buff_hex(bData);
					}
					else if(mon_hex_buff[1] == 1)		// 8Bit High Data Read
					{
						wData = PWORD(nAddr);
						bData = (BYTE)((wData >> 8) & 0xFF);
						push_tx_buff_hex(bData);
					}
					else if(mon_hex_buff[1] == 2)		// 16Bit Data Read
					{
						wData = PWORD(nAddr);
						push_tx_buff_hex((BYTE)(wData >> 8));
						push_tx_buff_hex((BYTE)(wData));
					}
					else if(mon_hex_buff[1] == 3)		// 32Bit Data Read
					{
						dwData = PLONG(nAddr);
						push_tx_buff_hex((BYTE)(dwData >> 24));
						push_tx_buff_hex((BYTE)(dwData >> 16));
						push_tx_buff_hex((BYTE)(dwData >> 8));
						push_tx_buff_hex((BYTE)(dwData));					
					}
					
					cksum = 0;
					for(i = 1; i < mon_tx_end; i++) cksum += mon_tx_buff[i];
					push_tx_buff_hex(cksum);
					push_tx_buff(END_CHAR);
				}
				else if(mon_hex_buff[0] == 0x52)	// Write Command
				{
					if(mon_hex_buff[1] == 0)				// 8Bit Low
					{
						bData = (BYTE)mon_hex_buff[5];
						wData = PWORD(nAddr);
						wData = ((wData & 0xFF00) | bData);
						PWORD(nAddr) = wData;
					}
					else if(mon_hex_buff[1] == 1)			// 8Bit High
					{
						bData = (BYTE)mon_hex_buff[5];
						wData = PWORD(nAddr);
						wData = ((wData & 0xFF) | (bData << 8));
						PWORD(nAddr) = wData;
					}
					else if(mon_hex_buff[1] == 2)			// 16Bit Data
					{
						wData = (WORD)(mon_hex_buff[5] << 8);
						wData |= (WORD)mon_hex_buff[6];
						PWORD(nAddr) = (wData & 0xFFFF);
						
					}
					else if(mon_hex_buff[1] == 3)			// 32Bit Data
					{						
						dwData = ((DWORD)(mon_hex_buff[5] & 0xFF) << 24);
						dwData |= ((DWORD)(mon_hex_buff[6] & 0xFF) << 16);
						dwData |= ((DWORD)(mon_hex_buff[7] & 0xFF) << 8);
						dwData |= ((DWORD)mon_hex_buff[8] & 0xFF);
						PLONG(nAddr) = dwData;						
					}
					
					push_tx_buff(START_CHAR);
					push_tx_buff_hex(0x52);
					cksum = 0;
					for(i = 1; i < mon_tx_end; i++) cksum += mon_tx_buff[i];
					push_tx_buff_hex(cksum);
					push_tx_buff(END_CHAR);
				}
				SCI0_TX_START;				
			}
		}				
	}
}

#pragma interrupt alignsp
void SCI0_TX_EMPTY(void)
{
unsigned short temp;

	temp = SCI0_SR;
	if(mon_tx_pos != mon_tx_end) SCI0_DR = mon_tx_buff[mon_tx_pos++];
	else SCI0_TX_STOP;
}